import { __decorate } from "tslib";
import * as PDFjs from "pdfjs-dist/legacy/build/pdf.js";
import * as XmlCore from "xml-core";
import * as XAdES from "xadesjs";
import { TrustedList } from "../tl";
export class AATL {
    constructor({ url = AATL.URL } = {}) {
        this.url = url;
    }
    async extractFile() {
        const doc = await PDFjs.getDocument(this.url).promise;
        const attachments = await doc.getAttachments();
        const attachmentName = "SecuritySettings.xml";
        const attachment = attachments[attachmentName];
        if (!attachment) {
            throw new Error(`Attachment ${attachmentName} not found in the source document`);
        }
        if (!attachment.content) {
            throw new Error(`Attachment ${attachmentName} content is unavailable or invalid`);
        }
        const xml = Buffer.from(attachment.content).toString("utf8");
        return xml;
    }
    async getTrusted(data) {
        var _a, _b;
        if (!data) {
            data = await this.extractFile();
        }
        const parsed = XAdES.Parse(data);
        if (!(XmlCore.isElement(parsed.lastChild))) {
            throw new Error("AATL: Invalid XML structure");
        }
        const settings = SecuritySettings.LoadXml(parsed.lastChild);
        let tl = new TrustedList();
        const trustItems = [
            "ROOT",
            "CERTIFIED_DOCUMENTS",
            "DYNAMIC_CONTENT",
            "JAVASCRIPT",
        ];
        for (const identity of settings.TrustedIdentities.GetIterator()) {
            if (identity.Trust === null || identity.Certificate === null) {
                continue;
            }
            if (identity.Identification.Source !== "AATL") {
                continue;
            }
            const trust = [];
            for (const trustName of trustItems) {
                const localName = XmlSecuritySettings.TrustTypeMapping[trustName];
                if (identity.Trust[localName] === true) {
                    trust.push(trustName);
                }
            }
            const policyArray = ((_b = (_a = identity.PolicyRestrictions) === null || _a === void 0 ? void 0 : _a.CertificatePolicies) === null || _b === void 0 ? void 0 : _b.GetIterator()) || [];
            const evpolicy = policyArray.map(oid => oid.Value);
            tl.AddCertificate({
                trust,
                evpolicy,
                operator: "Unknown",
                raw: identity.Certificate,
                source: "AATL"
            });
        }
        return tl;
    }
    async getDisallowed(data) {
        var _a, _b, _c;
        if (!data) {
            data = await this.extractFile();
        }
        const parsed = XAdES.Parse(data);
        if (!(XmlCore.isElement(parsed.lastChild))) {
            throw new Error("AATL: Invalid XML structure");
        }
        const settings = SecuritySettings.LoadXml(parsed.lastChild);
        let tl = new TrustedList();
        for (const identity of settings.TrustedIdentities.GetIterator()) {
            if (identity.ImportAction !== 3) {
                continue;
            }
            if (((_a = identity.Identification) === null || _a === void 0 ? void 0 : _a.Source) !== "AATL") {
                continue;
            }
            const evpolicy = ((_c = (_b = identity.PolicyRestrictions) === null || _b === void 0 ? void 0 : _b.CertificatePolicies) === null || _c === void 0 ? void 0 : _c.GetIterator().map((oid) => oid.Value)) || [];
            tl.AddCertificate({
                trust: [],
                evpolicy,
                raw: identity.Certificate,
                source: "AATL"
            });
        }
        return tl;
    }
}
AATL.URL = "https://trustlist.adobe.com/tl12.acrobatsecuritysettings";
const XmlSecuritySettings = {
    ElementNames: {
        SecuritySettings: "SecuritySettings",
        TrustedIdentities: "TrustedIdentities",
        Identity: "Identity",
        ImportAction: "ImportAction",
        Certificate: "Certificate",
        Trust: "Trust",
        Root: "Root",
        CertifiedDocuments: "CertifiedDocuments",
        DynamicContent: "DynamicContent",
        JavaScript: "JavaScript",
        SystemOperations: "SystemOperations",
        CrossDomain: "CrossDomain",
        ExternalStream: "ExternalStream",
        SilentPrint: "SilentPrint",
        WebLink: "WebLink",
        DataInjection: "DataInjection",
        ScriptInjection: "ScriptInjection",
        PolicyRestrictions: "PolicyRestrictions",
        NonExplicitProcessing: "NonExplicitProcessing",
        Description: "Description",
        CertificatePolicies: "CertificatePolicies",
        OID: "OID",
        Identification: "Identification",
        Source: "Source",
        Visible: "Visible",
    },
    TrustTypeMapping: {
        ROOT: "Root",
        CERTIFIED_DOCUMENTS: "CertifiedDocuments",
        DYNAMIC_CONTENT: "DynamicContent",
        JAVASCRIPT: "JavaScript",
    }
};
let SystemOperations = class SystemOperations extends XmlCore.XmlObject {
    constructor() {
        super(...arguments);
        this.CrossDomain = null;
        this.ExternalStream = null;
        this.SilentPrint = null;
        this.WebLink = null;
        this.DataInjection = null;
        this.ScriptInjection = null;
    }
};
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.CrossDomain)
], SystemOperations.prototype, "CrossDomain", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.ExternalStream)
], SystemOperations.prototype, "ExternalStream", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.SilentPrint)
], SystemOperations.prototype, "SilentPrint", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.WebLink)
], SystemOperations.prototype, "WebLink", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.DataInjection)
], SystemOperations.prototype, "DataInjection", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.ScriptInjection)
], SystemOperations.prototype, "ScriptInjection", void 0);
SystemOperations = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.SystemOperations })
], SystemOperations);
let Trust = class Trust extends XmlCore.XmlObject {
    constructor() {
        super(...arguments);
        this.Root = null;
        this.CertifiedDocuments = null;
        this.DynamicContent = null;
        this.JavaScript = null;
        this.SystemOperations = null;
    }
};
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.Root)
], Trust.prototype, "Root", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.CertifiedDocuments)
], Trust.prototype, "CertifiedDocuments", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.DynamicContent)
], Trust.prototype, "DynamicContent", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.JavaScript)
], Trust.prototype, "JavaScript", void 0);
__decorate([
    XmlCore.XmlChildElement({
        localName: XmlSecuritySettings.ElementNames.SystemOperations,
        maxOccurs: 1,
        parser: SystemOperations
    })
], Trust.prototype, "SystemOperations", void 0);
Trust = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.Trust })
], Trust);
function uniqueFlag(name) {
    return XmlCore.XmlChildElement({
        localName: name,
        maxOccurs: 1,
        converter: {
            get: (value) => {
                if (value) {
                    return "1";
                }
                return "0";
            },
            set: (value) => {
                switch (value) {
                    case "1":
                        return true;
                    case "0":
                        return false;
                    default:
                        console.warn(`AATL: WARNING, invalid value for binary flag ${name}: "${value}"`);
                        return false;
                }
            },
        }
    });
}
let OID = class OID extends XmlCore.XmlObject {
    constructor() {
        super(...arguments);
        this.Value = null;
    }
};
__decorate([
    XmlCore.XmlContent({ required: true })
], OID.prototype, "Value", void 0);
OID = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.OID })
], OID);
let CertificatePolicies = class CertificatePolicies extends XmlCore.XmlCollection {
};
CertificatePolicies = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.CertificatePolicies, parser: OID })
], CertificatePolicies);
let PolicyRestrictions = class PolicyRestrictions extends XmlCore.XmlObject {
    constructor() {
        super(...arguments);
        this.NonExplicitProcessing = null;
        this.Description = null;
        this.CertificatePolicies = null;
    }
};
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.NonExplicitProcessing)
], PolicyRestrictions.prototype, "NonExplicitProcessing", void 0);
__decorate([
    XmlCore.XmlChildElement({ localName: XmlSecuritySettings.ElementNames.Description, maxOccurs: 1 })
], PolicyRestrictions.prototype, "Description", void 0);
__decorate([
    XmlCore.XmlChildElement({ localName: XmlSecuritySettings.ElementNames.CertificatePolicies, maxOccurs: 1, parser: CertificatePolicies })
], PolicyRestrictions.prototype, "CertificatePolicies", void 0);
PolicyRestrictions = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.PolicyRestrictions })
], PolicyRestrictions);
let Identification = class Identification extends XmlCore.XmlObject {
    constructor() {
        super(...arguments);
        this.Source = null;
        this.Visible = null;
    }
};
__decorate([
    XmlCore.XmlChildElement({ localName: XmlSecuritySettings.ElementNames.Source, maxOccurs: 1 })
], Identification.prototype, "Source", void 0);
__decorate([
    uniqueFlag(XmlSecuritySettings.ElementNames.Visible)
], Identification.prototype, "Visible", void 0);
Identification = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.Identification })
], Identification);
let Identity = class Identity extends XmlCore.XmlObject {
    constructor() {
        super(...arguments);
        this.ImportAction = null;
        this.Certificate = null;
        this.Trust = null;
        this.PolicyRestrictions = null;
        this.Identification = null;
    }
};
__decorate([
    XmlCore.XmlChildElement({
        localName: XmlSecuritySettings.ElementNames.ImportAction,
        maxOccurs: 1,
        converter: XmlCore.XmlNumberConverter,
    })
], Identity.prototype, "ImportAction", void 0);
__decorate([
    XmlCore.XmlChildElement({
        localName: XmlSecuritySettings.ElementNames.Certificate,
        maxOccurs: 1,
    })
], Identity.prototype, "Certificate", void 0);
__decorate([
    XmlCore.XmlChildElement({
        localName: XmlSecuritySettings.ElementNames.Trust,
        maxOccurs: 1,
        parser: Trust,
    })
], Identity.prototype, "Trust", void 0);
__decorate([
    XmlCore.XmlChildElement({
        localName: XmlSecuritySettings.ElementNames.PolicyRestrictions,
        maxOccurs: 1,
        parser: PolicyRestrictions,
    })
], Identity.prototype, "PolicyRestrictions", void 0);
__decorate([
    XmlCore.XmlChildElement({
        localName: XmlSecuritySettings.ElementNames.Identification,
        maxOccurs: 1,
        parser: Identification,
    })
], Identity.prototype, "Identification", void 0);
Identity = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.Identity })
], Identity);
let TrustedIdentities = class TrustedIdentities extends XmlCore.XmlCollection {
};
TrustedIdentities = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.TrustedIdentities, parser: Identity })
], TrustedIdentities);
let SecuritySettings = class SecuritySettings extends XmlCore.XmlObject {
    constructor() {
        super(...arguments);
        this.TrustedIdentities = null;
    }
};
__decorate([
    XmlCore.XmlChildElement({
        parser: TrustedIdentities,
        required: true,
    })
], SecuritySettings.prototype, "TrustedIdentities", void 0);
SecuritySettings = __decorate([
    XmlCore.XmlElement({ localName: XmlSecuritySettings.ElementNames.SecuritySettings })
], SecuritySettings);
